@@ -255,7 +255,8 @@ class SalePromotion(ModelSQL, ModelView, MatchMixin):
         # Apply promotion only if all unit prices decrease
         if all(l.unit_price >= p for l, p in new_prices.iteritems()):
             for line, unit_price in new_prices.iteritems():
-                line.unit_price = unit_price
+                line.unit_price = unit_price.quantize(
+                    Decimal(1) / 10 ** line.__class__.unit_price.digits[1])
                 line.promotion = self
             sale.lines = sale.lines  # Trigger the change
 
@@ -263,8 +264,8 @@ class SalePromotion(ModelSQL, ModelView, MatchMixin):
         pool = Pool()
         Product = pool.get('product.product')
         if sale_line:
-            with Transaction().set_context(uom=sale_line.unit.id,
-                    currency=sale_line.sale.currency.id):
+            with Transaction().set_context(
+                    sale_line._get_context_sale_price()):
                 prices = Product.get_sale_price([sale_line.product])
             unit_price = prices[sale_line.product.id]
         else:
